@@ -1,21 +1,23 @@
 #------------------------------------------------------------------------------
 # File:         Photoshop.pm
 #
-# Description:  Read/Write Photoshop IRB meta information
+# Description:  Read/write Photoshop IRB meta information
 #
-# Revisions:    02/06/04 - P. Harvey Created
-#               02/25/04 - P. Harvey Added hack for problem with old photoshops
-#               10/04/04 - P. Harvey Added a bunch of tags (ref Image::MetaData::JPEG)
-#                          but left most of them commented out until I have enough
-#                          information to write PrintConv routines for them to
-#                          display something useful
-#               07/08/05 - P. Harvey Added support for reading PSD files
-#               01/07/06 - P. Harvey Added PSD write support
+# Revisions:    02/06/2004 - P. Harvey Created
+#               02/25/2004 - P. Harvey Added hack for problem with old photoshops
+#               10/04/2004 - P. Harvey Added a bunch of tags (ref Image::MetaData::JPEG)
+#                            but left most of them commented out until I have enough
+#                            information to write PrintConv routines for them to
+#                            display something useful
+#               07/08/2005 - P. Harvey Added support for reading PSD files
+#               01/07/2006 - P. Harvey Added PSD write support
+#               11/04/2006 - P. Harvey Added handling of resource name
 #
 # References:   1) http://www.fine-view.com/jp/lab/doc/ps6ffspecsv2.pdf
 #               2) http://www.ozhiker.com/electronics/pjmt/jpeg_info/irb_jpeg_qual.html
 #               3) Matt Mueller private communication (tests with PS CS2)
 #               4) http://www.fileformat.info/format/psd/egff.htm
+#               5) http://www.telegraphics.com.au/svn/psdparse/trunk/resources.c
 #------------------------------------------------------------------------------
 
 package Image::ExifTool::Photoshop;
@@ -24,7 +26,7 @@ use strict;
 use vars qw($VERSION $AUTOLOAD);
 use Image::ExifTool qw(:DataAccess);
 
-$VERSION = '1.27';
+$VERSION = '1.36';
 
 sub ProcessPhotoshop($$$);
 sub WritePhotoshop($$$);
@@ -54,7 +56,7 @@ my %psdMap = (
     WRITE_PROC => \&WritePhotoshop,
     0x03e8 => { Unknown => 1, Name => 'Photoshop2Info' },
     0x03e9 => { Unknown => 1, Name => 'MacintoshPrintInfo' },
-    0x03ea => { Unknown => 1, Name => 'XMLData' }, #PH
+    0x03ea => { Unknown => 1, Name => 'XMLData', Binary => 1 }, #PH
     0x03eb => { Unknown => 1, Name => 'Photoshop2ColorTable' },
     0x03ed => {
         Name => 'ResolutionInfo',
@@ -102,7 +104,11 @@ my %psdMap = (
         },
     },
     0x0408 => { Unknown => 1, Name => 'GridGuidesInfo' },
-    0x0409 => { Unknown => 1, Name => 'ThumbnailResource' },
+    0x0409 => {
+        Name => 'PhotoshopBGRThumbnail',
+        Notes => 'this is a JPEG image, but in BGR format instead of RGB',
+        RawConv => 'my $img=substr($val,0x1c);$self->ValidateImage(\$img,$tag)',
+    },
     0x040a => {
         Name => 'CopyrightFlag',
         Writable => 'int8u',
@@ -121,7 +127,7 @@ my %psdMap = (
     },
     0x040c => {
         Name => 'PhotoshopThumbnail',
-        ValueConv => 'my $img=substr($val,0x1c);$self->ValidateImage(\$img,$tag)',
+        RawConv => 'my $img=substr($val,0x1c);$self->ValidateImage(\$img,$tag)',
     },
     0x040d => {
         Name => 'GlobalAngle',
@@ -164,12 +170,40 @@ my %psdMap = (
             WriteProc => \&Image::ExifTool::WriteTIFF,
         },
     },
+    0x0423 => { Unknown => 1, Name => 'ExifInfo2', Binary => 1 }, #5
     0x0424 => {
         Name => 'XMP',
         SubDirectory => {
             TagTable => 'Image::ExifTool::XMP::Main',
         },
     },
+    0x0425 => {
+        Name => 'IPTCDigest',
+        Writable => 'string',
+        Protected => 1,
+        Notes => q{
+            when writing, a special value of "new" represents the new IPTC digest of the
+            output file, or deletes this tag if no IPTC exists
+        },
+        # also note the 'new' feature requires that the IPTC comes before this tag is written
+        ValueConv => 'unpack("H*", $val)',
+        ValueConvInv => q{
+            if (lc($val) eq 'new') {
+                return 'new' if eval 'require Digest::MD5';
+                warn "Digest::MD5 must be installed\n";
+                return undef;
+            }
+            return pack('H*', $val) if $val =~ /^[0-9a-f]{32}$/i;
+            warn "Value must be 32 hexadecimal digits or 'new'\n";
+            return undef;
+        }
+    },
+    0x0426 => { Unknown => 1, Name => 'PrintScale' }, #5
+    0x0428 => { Unknown => 1, Name => 'PixelAspectRatio' }, #5
+    0x0429 => { Unknown => 1, Name => 'LayerComps' }, #5
+    0x042a => { Unknown => 1, Name => 'AlternateDuotoneColors' }, #5
+    0x042b => { Unknown => 1, Name => 'AlternateSpotColors' }, #5
+    # 0x07d0-0x0bb6 Path information
     0x0bb7 => { Unknown => 1, Name => 'ClippingPathName' },
     0x2710 => { Unknown => 1, Name => 'PrintFlagsInfo' },
 );
@@ -273,6 +307,11 @@ my %psdMap = (
     },
 );
 
+# tags for unknown resource types
+%Image::ExifTool::Photoshop::Unknown = (
+    GROUPS => { 2 => 'Unknown' },
+);
+
 #------------------------------------------------------------------------------
 # AutoLoad our writer routines when necessary
 #
@@ -319,22 +358,30 @@ sub ProcessPhotoshop($$$)
     $verbose and $exifTool->VerboseDir('Photoshop', 0, $$dirInfo{DirLen});
 
     # scan through resource blocks:
-    # Format: 0) Type, 4 bytes - "8BIM"
+    # Format: 0) Type, 4 bytes - '8BIM' (or the rare 'PHUT', 'DCSR' or 'AgHg')
     #         1) TagID,2 bytes
     #         2) Name, pascal string padded to even no. bytes
     #         3) Size, 4 bytes - N
     #         4) Data, N bytes
     while ($pos + 8 < $dirEnd) {
         my $type = substr($$dataPt, $pos, 4);
-        if ($type ne '8BIM') {
-            $exifTool->Warn("Bad Photoshop IRB resource");
+        my ($ttPtr, $extra, $val, $name);
+        if ($type eq '8BIM') {
+            $ttPtr = $tagTablePtr;
+        } elsif ($type =~ /^(PHUT|DCSR|AgHg)$/) {
+            $ttPtr = Image::ExifTool::GetTagTable('Image::ExifTool::Photoshop::Unknown');
+        } else {
+            $type =~ s/([^\w])/sprintf("\\x%.2x",ord($1))/ge;
+            $exifTool->Warn(qq{Bad Photoshop IRB resource "$type"});
             last;
         }
         my $tag = Get16u($dataPt, $pos + 4);
-        my $namelen = 1 + Get8u($dataPt, $pos + 6);
-        ++$namelen if $namelen & 0x01;
+        $pos += 6;  # point to start of name
+        my $nameLen = Get8u($dataPt, $pos);
+        my $namePos = ++$pos;
         # skip resource block name (pascal string, padded to an even # of bytes)
-        $pos += 6 + $namelen;
+        $pos += $nameLen;
+        ++$pos unless $nameLen & 0x01;
         if ($pos + 4 > $dirEnd) {
             $exifTool->Warn("Bad Photoshop resource block");
             last;
@@ -346,7 +393,22 @@ sub ProcessPhotoshop($$$)
             last;
         }
         $success = 1;
-        $exifTool->HandleTag($tagTablePtr, $tag, undef,
+        if ($nameLen) {
+            $name = substr($$dataPt, $namePos, $nameLen);
+            $extra = qq{, Name="$name"};
+        } else {
+            $name = '';
+        }
+        my $tagInfo = $exifTool->GetTagInfo($ttPtr, $tag);
+        # append resource name to value if requested (braced by "/#...#/")
+        if ($tagInfo and defined $$tagInfo{SetResourceName} and
+            $$tagInfo{SetResourceName} eq '1' and $name !~ m{/#})
+        {
+            $val = substr($$dataPt, $pos, $size) . '/#' . $name . '#/';
+        }
+        $exifTool->HandleTag($ttPtr, $tag, $val,
+            TagInfo => $tagInfo,
+            Extra   => $extra,
             DataPt  => $dataPt,
             DataPos => $$dirInfo{DataPos},
             Size    => $size,
@@ -405,9 +467,37 @@ sub ProcessPSD($$)
         if ($data) {
             $len = Set32u(length $data);
             Write($outfile, $len, $data) or $err = 1;
-            # copy over the rest of the file
-            while ($raf->Read($data, 65536)) {
-                Write($outfile, $data) or $err = 1;
+            # look for trailer and edit if necessary
+            my $trailInfo = Image::ExifTool::IdentifyTrailer($raf);
+            if ($trailInfo) {
+                my $tbuf = '';
+                $$trailInfo{OutFile} = \$tbuf;  # rewrite trailer(s)
+                # rewrite all trailers to buffer
+                if ($exifTool->ProcessTrailers($trailInfo)) {
+                    my $copyBytes = $$trailInfo{DataPos} - $raf->Tell();
+                    if ($copyBytes >= 0) {
+                        # copy remaining PSD file up to start of trailer
+                        while ($copyBytes) {
+                            my $n = ($copyBytes > 65536) ? 65536 : $copyBytes;
+                            $raf->Read($data, $n) == $n or $err = 1;
+                            Write($outfile, $data) or $err = 1;
+                            $copyBytes -= $n;
+                        }
+                        # write the trailer (or not)
+                        $exifTool->WriteTrailerBuffer($trailInfo, $outfile) or $err = 1;
+                    } else {
+                        $exifTool->Warn('Overlapping trailer');
+                        undef $trailInfo;
+                    }
+                } else {
+                    undef $trailInfo;
+                }
+            }
+            unless ($trailInfo) {
+                # copy over the rest of the file
+                while ($raf->Read($data, 65536)) {
+                    Write($outfile, $data) or $err = 1;
+                }
             }
         } else {
             $err = 1;
@@ -417,6 +507,9 @@ sub ProcessPSD($$)
         $exifTool->Warn('File format error');
     } else {
         ProcessPhotoshop($exifTool, \%dirInfo, $tagTablePtr);
+        # process trailers if they exist
+        my $trailInfo = Image::ExifTool::IdentifyTrailer($raf);
+        $exifTool->ProcessTrailers($trailInfo) if $trailInfo;
     }
     return $rtnVal;
 }
@@ -428,7 +521,7 @@ __END__
 
 =head1 NAME
 
-Image::ExifTool::Photoshop - Read/Write Photoshop IRB meta information
+Image::ExifTool::Photoshop - Read/write Photoshop IRB meta information
 
 =head1 SYNOPSIS
 
@@ -440,9 +533,21 @@ Photoshop writes its own format of meta information called a Photoshop IRB
 resource which is located in the APP13 record of JPEG files.  This module
 contains the definitions to read this information.
 
+=head1 NOTES
+
+Photoshop IRB blocks may have an associated resource name.  These names are
+usually just an empty string, but if not empty they are displayed in the
+verbose level 2 (or greater) output.  A special C<SetResourceName> flag may
+be set to '1' in the tag information hash to cause the resource name to be
+appended to the value when extracted.  If this is done, the returned value
+has the form "VALUE/#NAME#/".  When writing, the writer routine looks for
+this syntax (if C<SetResourceName> is defined), and and uses the embedded
+name to set the name of the new resource.  This allows the resource names to
+be preserved when copying Photoshop information via user-defined tags.
+
 =head1 AUTHOR
 
-Copyright 2003-2006, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2008, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
